@@ -13,6 +13,7 @@ import json
 import logging
 import os
 import shutil
+import signal
 import tarfile
 import tempfile
 
@@ -20,19 +21,25 @@ from contextlib import contextmanager
 from urllib.parse import urlparse
 
 from .. _internal import blkhash
-from .. _internal import io
 from .. _internal import qemu_img
 from .. _internal import qemu_nbd
 from .. _internal.backends import http, nbd
 from .. _internal.handlers import checksum as _checksum
 from .. _internal.nbd import UnixAddress
 
+from . import _app
+from . import _io
+
+# API constants.
+BUFFER_SIZE = _io.BUFFER_SIZE
+MAX_WORKERS = _io.MAX_WORKERS
+
 log = logging.getLogger("client")
 
 
-def upload(filename, url, cafile, buffer_size=io.BUFFER_SIZE, secure=True,
-           progress=None, proxy_url=None, max_workers=io.MAX_WORKERS,
-           member=None, backing_chain=True):
+def upload(filename, url, cafile, buffer_size=BUFFER_SIZE, secure=True,
+           progress=None, proxy_url=None, max_workers=MAX_WORKERS,
+           member=None, backing_chain=True, disk_is_zero=False):
     """
     Upload filename to url
 
@@ -62,6 +69,10 @@ def upload(filename, url, cafile, buffer_size=io.BUFFER_SIZE, secure=True,
             image data, leaving unallocated areas as holes, exposing data from
             the target disk backing chain. Valid only when uploding to an empty
             snapshot.
+        disk_is_zero (bool): If set, skip zero extents in the source image
+            instead of zeroing the extent in the destination disk. Should be
+            used only when uploading to new disk on file based storage or new
+            qcow2 disk on block based storage.
     """
     if callable(progress):
         progress = ProgressWrapper(progress)
@@ -91,15 +102,16 @@ def upload(filename, url, cafile, buffer_size=io.BUFFER_SIZE, secure=True,
                 backing_chain=backing_chain) as src:
 
             # Upload the image to the server.
-            io.copy(
+            _io.copy(
                 src,
                 dst,
                 max_workers=max_workers,
                 buffer_size=buffer_size,
-                # Since we don't know if the destination image is empty, we
-                # always want to zero. This can be optimized if the caller
-                # knows that the image is empty.
-                zero=True,
+                # If the disk is zero and we upload the entire chain, we can
+                # skip zero extents instead of zeroing the area on the
+                # destination.  When uploading without a backing chain we must
+                # zero extents which are zero but not holes.
+                zero=not (disk_is_zero and backing_chain),
                 # When uploading without a backing chain, the destination image
                 # has a backing chain. We must keep holes unallocated on the so
                 # they expose data from the backing chain.
@@ -109,8 +121,8 @@ def upload(filename, url, cafile, buffer_size=io.BUFFER_SIZE, secure=True,
 
 
 def download(url, filename, cafile, fmt="qcow2", incremental=False,
-             buffer_size=io.BUFFER_SIZE, secure=True, progress=None,
-             proxy_url=None, max_workers=io.MAX_WORKERS,
+             buffer_size=BUFFER_SIZE, secure=True, progress=None,
+             proxy_url=None, max_workers=MAX_WORKERS,
              backing_file=None, backing_format=None):
     """
     Download url to filename.
@@ -169,7 +181,7 @@ def download(url, filename, cafile, fmt="qcow2", incremental=False,
         with _open_nbd(filename, fmt, shared=max_workers) as dst:
 
             # Download the image from the server to the local image.
-            io.copy(
+            _io.copy(
                 src,
                 dst,
                 dirty=incremental,
@@ -288,7 +300,7 @@ class ImageioClient:
     """
 
     def __init__(self, transfer_url, cafile=None, secure=True, proxy_url=None,
-                 buffer_size=io.BUFFER_SIZE):
+                 buffer_size=BUFFER_SIZE):
         """
         Arguments:
             transfer_url (str): Transfer url on the host running imageio server
@@ -483,6 +495,11 @@ def _open_nbd(filename, fmt, read_only=False, shared=1, bitmap=None,
     """
     with _tmp_dir("imageio-") as base:
         sock = UnixAddress(os.path.join(base, "sock"))
+
+        # If the applicatiion is handling signals, block SIGINT in qemu-nbd for
+        # clean termination when receiving SIGINT.
+        signals = {signal.SIGINT} if _app.is_handling_signals() else None
+
         with qemu_nbd.run(
                 filename,
                 fmt,
@@ -492,7 +509,8 @@ def _open_nbd(filename, fmt, read_only=False, shared=1, bitmap=None,
                 bitmap=bitmap,
                 offset=offset,
                 size=size,
-                backing_chain=backing_chain):
+                backing_chain=backing_chain,
+                block_signals=signals):
             url = urlparse(sock.url())
             mode = "r" if read_only else "r+"
             yield nbd.open(url, mode=mode, dirty=bitmap is not None)
